-- *****************************************************
-- Variables
-- *****************************************************
-- /run print(GetCurrentMapAreaID())
-- Hellfire Citadel   : 1026
-- Blackrock Furnance : 988
-- Highmaul           : 994
local WodHMId, WodBRFId, WodHFCId = 994, 988, 1026
local RaidDiffcultyNormal, RaidDiffcultyHeroic, RaidDiffcultyMythic = "Normal", "Heroic", "Mythic"

local FlasksId = {}
FlasksId[1] = 156084 -- Greater Draenic Stamina Flask
FlasksId[2] = 156079 -- Greater Draenic Intellect Flask	
FlasksId[3] = 156064 -- Greater Draenic Agility Flask
FlasksId[4] = 156080 -- Greater Draenic Strength Flask

local PotionsId = {}
PotionsId[1] = 156426 -- Draenic Intellect Potion
PotionsId[2] = 156423 -- Draenic Agility Potion
PotionsId[3] = 156428 -- Draenic Strength Potion
PotionsId[4] = 156430 -- Draenic Armor Potion
PotionsId[5] = 156436 -- Draenic Mana Potion
PotionsId[6] = 156432 -- Draenic Channeled Mana Potion

local Foods100Id = {}
Foods100Id[1] = 175218 -- Critical Strike
Foods100Id[2] = 175223 -- Versatility
Foods100Id[3] = 175219 -- Haste
Foods100Id[4] = 175222 -- Multistrike
Foods100Id[5] = 175220 -- Mastery

local Foods125Id = {}
Foods125Id[1] = 180745 -- Critical Strike
Foods125Id[2] = 180746 -- Versatility
Foods125Id[3] = 180747 -- Stamina
Foods125Id[4] = 180748 -- Haste
Foods125Id[5] = 180749 -- Multistrike
Foods125Id[6] = 180750 -- Mastery

local RunesId = {}
RunesId[1] = 175457 -- Focus Augmentation - Intellect
RunesId[2] = 175456 -- Hyper Augmentation - Agility
RunesId[3] = 175439 -- Stout Augmentation - Strength



-- *****************************************************
-- Slash Commands
-- *****************************************************
SLASH_HUNGRYRAID1 = '/hr'
function SlashCmdList.HUNGRYRAID(msg, editbox)
	if (msg == "options") then
		InterfaceOptionsFrame_OpenToCategory(frmHungryRaid)
		InterfaceOptionsFrame_OpenToCategory(frmHungryRaid)
	end
	
	if (msg == "help" or msg == "") then
		HungryRaidHelp()
	end
	if (msg == "last") then
		if (frmResult ~= nil) then
			frmResult:Show()
		else
			print("Hungry Raid - There are no fight logged")
		end
	end

end

function HungryRaidHelp()
		print("========================================")
		print("| Hungry Raid Help")
		print("========================================")
		print("| /hr help: Show this help")
		print("| /hr options: Show options frame")
		print("| /hr last: show last fight result")
		print("========================================")
end

-- *****************************************************
-- Load
-- *****************************************************
function HungryRaid_Load(panel)
	panel.name = 'Hungry Raid'
	panel.okay = function (self) ConsumablesSave(panel); end;
	panel.cancel = function (self) ConsumablesCancel(panel);  end;
	InterfaceOptions_AddCategory(panel)
	MakeOptionsUI(panel)

	-- Events
	panel:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	panel:RegisterEvent("ADDON_LOADED")
	panel:RegisterEvent("ENCOUNTER_START")
	panel:RegisterEvent("ENCOUNTER_END")
	panel:RegisterEvent("PLAYER_LOGOUT")
	panel:RegisterEvent("VARIABLES_LOADED")
	
	panel:SetScript("OnEvent", EventListner)
	
end

function EndFightCallBack()
	if (CheckValidCombat()) then
		ListResults()
	end
end

function ConsumablesSave(panel)
	SaveOptions()
end

function ConsumablesCancel(panel)
	LoadSavedOptions()
end

-- *****************************************************
-- Event Listener
-- *****************************************************
function EventListner(self, event, addonName, eventType, ...)
	if (event == 'ADDON_LOADED' and addonName == 'HungryRaid') then
		LoadSavedOptions()
		SetFramesVisibility(self)
		HungryRaidHelp()
	end

	if (HungryRaidDB["combat"] == nil) then
		HungryRaidDB["combat"] = {}
		HungryRaidDB["combat"].InEncounter = false
	end
	
	if (event == 'PLAYER_LOGOUT') then
		SaveOptions()
	end
	
	if (event == 'COMBAT_LOG_EVENT_UNFILTERED') then
		if (HungryRaidDB["combat"].InEncounter) then
			if (eventType == 'SPELL_AURA_APPLIED' or eventType == 'SPELL_ENERGIZE' or eventType == 'SPELL_PERIODIC_ENERGIZE') then
				CheckPotionUse(...)
			end
		end
	end
	
	if (event == 'ENCOUNTER_START') then
		frmResult = nil
		HungryRaidDB["combat"] = {}
		HungryRaidDB["combat"].InEncounter = true
		if (CheckValidCombat()) then
			LoadPlayers()
			CheckPreBuffs()
		end
	end
	
	if (event == 'ENCOUNTER_END') then
		if (CheckValidCombat()) then
			ListResults()
		end
		HungryRaidDB["combat"].InEncounter = false
	end
end

function CheckValidCombat()
	HungryRaidDB["combat"].Map = false
	HungryRaidDB["combat"].Diffculty = false
	HungryRaidDB["combat"].BossFight = false
	local map = GetCurrentMapAreaID()
	local name, type, difficultyIndex, difficultyName, maxPlayers, dynamicDifficulty, isDynamic, mapID = GetInstanceInfo()
	if (HungryRaidDB["options"][map]) then
		if (HungryRaidDB["options"][map].Selected) then
			HungryRaidDB["combat"].Map = true
		else
			return false
		end

		if (HungryRaidDB["options"][map][difficultyName]) then
			HungryRaidDB["combat"].Diffculty = true
		else
			return false
		end
	end
	
	return true
end

function LoadPlayers()
	HungryRaidDB["players"] = {}
	local guild = GetGuildInfo("player")
	
	local ii = 1
	for i = 1, GetNumGroupMembers(), 1 do
		local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i)
		local playerGuild, grank, p3, p4 = GetGuildInfo(name)
		if (guild == playerGuild and string.find(name, "-") == nil) then
			HungryRaidDB["players"][ii] = {}
			HungryRaidDB["players"][ii].Name = name
			HungryRaidDB["players"][ii].Flask = false
			HungryRaidDB["players"][ii].Food = false
			HungryRaidDB["players"][ii].PrePotion = false
			HungryRaidDB["players"][ii].Potion = false
			HungryRaidDB["players"][ii].Runes = false
			ii = ii + 1
		end
	end
end

function CheckPreBuffs()
	for indPlayer = 1, table.getn(HungryRaidDB["players"]) do
		local indBuff = 1
			
		local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, 
			shouldConsolidate, spellId = UnitBuff(HungryRaidDB["players"][indPlayer].Name, indBuff)
		while name ~= nil do
			if (FindInTable(spellId, FlasksId)) then
				HungryRaidDB["players"][indPlayer].Flask = true
			end
			
			if (HungryRaidDB["options"]["HungryRaid"]["Foods"].Selected100) then
				if (FindInTable(spellId, Foods100Id) or FindInTable(spellId, Foods125Id)) then
					HungryRaidDB["players"][indPlayer].Food = true
				end
			end
			
			if (HungryRaidDB["options"]["HungryRaid"]["Foods"].Selected125) then
				if (FindInTable(spellId, Foods125Id)) then
					HungryRaidDB["players"][indPlayer].Food = true
				end
			end
			
			if (FindInTable(spellId, RunesId)) then
				HungryRaidDB["players"][indPlayer].Rune = true
			end
			
			if (FindInTable(spellId, PotionsId)) then
				HungryRaidDB["players"][indPlayer].PrePotion = true
			end
			
			indBuff = indBuff + 1
			name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, 
				shouldConsolidate, spellId = UnitBuff(HungryRaidDB["players"][indPlayer].Name, indBuff)
		end
	end
end

function FindInTable(id, arr)
	local ret = false
	for ind = 1, table.getn(arr) do
		if (id == arr[ind]) then
			ret = true
		end
	end
	return ret
end

function ListResults()
	local frame = MakeResultFrame()

	if (HungryRaidDB["players"]) then
		for i = 1, table.getn(HungryRaidDB["players"]) do
			frame.AddPlayer(
				frame, 
				HungryRaidDB["players"][i].Name, 
				HungryRaidDB["players"][i].Flask, 
				HungryRaidDB["players"][i].Food, 
				HungryRaidDB["players"][i].Rune, 
				HungryRaidDB["players"][i].PrePotion, 
				HungryRaidDB["players"][i].Potion
			)
		end
	end
end

function CreditEP()
	if (HungryRaidDB["players"]) then
		for i = 1, table.getn(HungryRaidDB["players"]) do
			-- /run EPGP:IncEPBy("Frostsoll", "Flask Use", 5, false, false)
			local ep = 0
			if (HungryRaidDB["players"][i].Flask) then
				ep = ep + HungryRaidDB["options"]["HungryRaid"]["Flasks"].EPValue
			end
			if (HungryRaidDB["players"][i].Food) then
				ep = ep + HungryRaidDB["options"]["HungryRaid"]["Foods"].EPValue
			end
			if (HungryRaidDB["players"][i].Rune) then
				ep = ep + HungryRaidDB["options"]["HungryRaid"]["Runes"].EPValue
			end
			if (HungryRaidDB["players"][i].PrePotion) then
				ep = ep + HungryRaidDB["options"]["HungryRaid"]["Potions"].EPValue
			end
			if (HungryRaidDB["players"][i].Potion) then
				ep = ep + HungryRaidDB["options"]["HungryRaid"]["Potions"].EPValue
			end
			local nm = EPGP:IncEPBy(HungryRaidDB["players"][i].Name, "Consumables Use", ep, true, false)
		end
	end
end

function ReportEP()
	if (HungryRaidDB["players"]) then
		local foodlevel = ""
		if (HungryRaidDB["options"]["HungryRaid"]["Foods"].Selected100) then
			foodlevel = "100+"
		else
			if (HungryRaidDB["options"]["HungryRaid"]["Foods"].Selected125) then
				foodlevel = "125+"
			end
		end
	
		local flasks = ""
		local foods = ""
		local runes = ""
		local prepotion = ""
		local potion = ""

		local flasksNames = ""
		local foodsNames = ""
		local runesNames = ""
		local prepotionNames = ""
		local potionNames = ""
		
		if (EPGP) then
			flasks = "Flask (".. HungryRaidDB["options"]["HungryRaid"]["Flasks"].EPValue .. "EP): "
			foods = "Food " .. foodlevel .. " (" .. HungryRaidDB["options"]["HungryRaid"]["Foods"].EPValue .. "EP): "
			runes = "Runes (" .. HungryRaidDB["options"]["HungryRaid"]["Runes"].EPValue .. "EP): "
			prepotion = "Previous Potion (" .. HungryRaidDB["options"]["HungryRaid"]["Potions"].EPValue .. "EP): "
			potion = "Combat Potion (" .. HungryRaidDB["options"]["HungryRaid"]["Potions"].EPValue .. "EP): "
		else
			flasks = "Flask: "
			foods = "Food: "
			runes = "Runes: "
			prepotion = "Previous Potion: "
			potion = "Combat Potion: "
			SendChatMessage("HungryRaid Addon. Report of the last fight: ", "RAID")
		end
		
		for i = 1, table.getn(HungryRaidDB["players"]) do
			if (HungryRaidDB["players"][i].Flask == false) then
				if (flasksNames ~= "") then
					flasksNames = flasksNames .. ", "
				end
				flasksNames = flasksNames .. HungryRaidDB["players"][i].Name
			end
			if (HungryRaidDB["players"][i].Food == false) then
				if (foodsNames ~= "") then
					foodsNames = foodsNames .. ", "
				end
				foodsNames = foodsNames .. HungryRaidDB["players"][i].Name
			end
			if (HungryRaidDB["players"][i].Rune == false) then
				if (runesNames ~= "") then
					runesNames = runesNames .. ", "
				end
				runesNames = runesNames .. HungryRaidDB["players"][i].Name
			end
			if (HungryRaidDB["players"][i].PrePotion == false) then
				if (prepotionNames ~= "") then
					prepotionNames = prepotionNames .. ", "
				end
				prepotionNames = prepotionNames .. HungryRaidDB["players"][i].Name
			end
			if (HungryRaidDB["players"][i].Potion == false) then
				if (potionNames ~= "") then
					potionNames = potionNames .. ", "
				end
				potionNames = potionNames .. HungryRaidDB["players"][i].Name
			end
		end
		if (EPGP) then
			SendChatMessage("HungryRaid Addon (EPGP Integration). List of players who will not receive points: ", "RAID")
		else
			SendChatMessage("HungryRaid Addon. Report of the last fight: ", "RAID")
		end
		
		flasks = flasks .. flasksNames
		foods = foods .. foodsNames
		runes = runes .. foodsNames
		prepotion = prepotion .. prepotionNames
		potion = potion .. potionNames
		
		SendChatMessage(flasks, "RAID")
		SendChatMessage(foods, "RAID")
		SendChatMessage(runes, "RAID")
		SendChatMessage(prepotion, "RAID")
		SendChatMessage(potion, "RAID")
	end
end

function CheckPotionUse(...)
	local player = select(3, ...)
	local spellId = select(10, ...)
	
	if (HungryRaidDB["players"]) then
		for i = 1, table.getn(HungryRaidDB["players"]) do
			if (HungryRaidDB["players"][i].Name == player) then
				if (FindInTable(spellId, PotionsId)) then
					HungryRaidDB["players"][i].Potion = true
				end
			end
		end
	end
end